/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Detail/CoreExport.h"

#include <memory>
#include <string>

namespace Zivid
{
    class ComputeDeviceImpl;
    class Application;

    /// <summary>Contains information about the compute device used by <see cref="Zivid::Application" />.</summary>
    class ComputeDevice
    {
    private:
        std::shared_ptr<ComputeDeviceImpl> m_impl;

    public:
        /// <summary>Constructor</summary>
        ZIVID_CORE_EXPORT ComputeDevice();
#ifndef NO_DOC
        ZIVID_CORE_EXPORT explicit ComputeDevice(std::unique_ptr<ComputeDeviceImpl> &&impl);
#endif
        /// <summary>Destructor</summary>
        ZIVID_CORE_EXPORT ~ComputeDevice();

        /// <summary>Copy constructor</summary>
        ZIVID_CORE_EXPORT ComputeDevice(const ComputeDevice &other);

        /// <summary>Assignment operator</summary>
        ZIVID_CORE_EXPORT ComputeDevice &operator=(const ComputeDevice &other);

        /// <summary>Move constructor</summary>
        ZIVID_CORE_EXPORT ComputeDevice(ComputeDevice &&other) noexcept;

        /// <summary>Move assignment operator</summary>
        ZIVID_CORE_EXPORT ComputeDevice &operator=(ComputeDevice &&other) noexcept;

        /// <summary>Returns the model name of this compute device</summary>
        ZIVID_CORE_EXPORT std::string model() const;

        /// <summary>Returns the vendor of this compute device</summary>
        ZIVID_CORE_EXPORT std::string vendor() const;

        /// <summary>Get string representation of the compute device</summary>
        /// <returns>Compute device info as string</returns>
        ZIVID_CORE_EXPORT std::string toString() const;

#ifndef NO_DOC
        ZIVID_CORE_EXPORT ComputeDeviceImpl &getImpl();
        ZIVID_CORE_EXPORT const ComputeDeviceImpl &getImpl() const;
#endif
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const ComputeDevice &computeDevice);
} // namespace Zivid
