/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <memory>
#include <stdexcept>
#include <utility>

// Todo(ZIVID-983): Make doxygen skip most content in this file, but not regular functions. Possibly by using NO_DOC

// ------------------------------------------------------------------------------------------------------------------ //
#define ZIVID_PIMPL_VALUE_SEMANTICS(ClassName, Attributes)                                                             \
public:                                                                                                                \
    Attributes ~ClassName();                                                                                           \
    Attributes ClassName(const ClassName &other);                                                                      \
    Attributes ClassName &operator=(const ClassName &other);                                                           \
    Attributes ClassName(ClassName &&other) noexcept;                                                                  \
    Attributes ClassName &operator=(ClassName &&other) noexcept;                                                       \
                                                                                                                       \
    Attributes explicit ClassName(std::unique_ptr<class ClassName##Impl> &&impl);                                      \
    Attributes explicit ClassName(class ClassName##Impl &&impl);                                                       \
    Attributes class ClassName##Impl &getImpl();                                                                       \
    Attributes const class ClassName##Impl &getImpl() const;                                                           \
                                                                                                                       \
private:                                                                                                               \
    std::unique_ptr<class ClassName##Impl> m_impl

#define ZIVID_PIMPL_VALUE_SEMANTICS_CPP(ClassName)                                                                     \
    ClassName::ClassName(std::unique_ptr<ClassName##Impl> &&impl)                                                      \
        : m_impl{ std::move(impl) }                                                                                    \
    {}                                                                                                                 \
                                                                                                                       \
    ClassName::ClassName(ClassName##Impl &&impl)                                                                       \
        : ClassName{ std::make_unique<ClassName##Impl>(std::move(impl)) }                                              \
    {}                                                                                                                 \
                                                                                                                       \
    ClassName::ClassName(const ClassName &other)                                                                       \
        : ClassName{ ClassName##Impl{ other.getImpl() } }                                                              \
    {}                                                                                                                 \
                                                                                                                       \
    ClassName &ClassName::operator=(const ClassName &other)                                                            \
    {                                                                                                                  \
        if(this != &other)                                                                                             \
        {                                                                                                              \
            *m_impl = other.getImpl();                                                                                 \
        }                                                                                                              \
        return *this;                                                                                                  \
    }                                                                                                                  \
                                                                                                                       \
    ClassName##Impl &ClassName::getImpl()                                                                              \
    {                                                                                                                  \
        return const_cast<ClassName##Impl &>(std::as_const(*this).getImpl());                                          \
    }                                                                                                                  \
                                                                                                                       \
    const ClassName##Impl &ClassName::getImpl() const                                                                  \
    {                                                                                                                  \
        if(m_impl == nullptr)                                                                                          \
        {                                                                                                              \
            throw std::runtime_error(#ClassName " is empty");                                                          \
        }                                                                                                              \
        return *m_impl;                                                                                                \
    }                                                                                                                  \
                                                                                                                       \
    ClassName::~ClassName() = default;                                                                                 \
    ClassName::ClassName(ClassName &&other) noexcept = default;                                                        \
    ClassName &ClassName::operator=(ClassName &&other) noexcept = default

// ------------------------------------------------------------------------------------------------------------------ //
#define ZIVID_PIMPL_MOVE_ONLY(ClassName, Attributes)                                                                   \
public:                                                                                                                \
    Attributes ~ClassName();                                                                                           \
    Attributes ClassName(const ClassName &other) = delete;                                                             \
    Attributes ClassName &operator=(const ClassName &other) = delete;                                                  \
    Attributes ClassName(ClassName &&other) noexcept;                                                                  \
    Attributes ClassName &operator=(ClassName &&other) noexcept;                                                       \
                                                                                                                       \
    Attributes explicit ClassName(std::unique_ptr<class ClassName##Impl> &&impl);                                      \
    Attributes class ClassName##Impl &getImpl();                                                                       \
    Attributes const class ClassName##Impl &getImpl() const;                                                           \
                                                                                                                       \
private:                                                                                                               \
    std::unique_ptr<class ClassName##Impl> m_impl

#define ZIVID_PIMPL_MOVE_ONLY_CPP(ClassName)                                                                           \
    ClassName::ClassName(std::unique_ptr<ClassName##Impl> &&impl)                                                      \
        : m_impl{ std::move(impl) }                                                                                    \
    {}                                                                                                                 \
                                                                                                                       \
    ClassName##Impl &ClassName::getImpl()                                                                              \
    {                                                                                                                  \
        return const_cast<ClassName##Impl &>(std::as_const(*this).getImpl());                                          \
    }                                                                                                                  \
                                                                                                                       \
    const ClassName##Impl &ClassName::getImpl() const                                                                  \
    {                                                                                                                  \
        if(m_impl == nullptr)                                                                                          \
        {                                                                                                              \
            throw std::runtime_error(#ClassName " is empty");                                                          \
        }                                                                                                              \
        return *m_impl;                                                                                                \
    }                                                                                                                  \
                                                                                                                       \
    ClassName::~ClassName() = default;                                                                                 \
    ClassName::ClassName(ClassName &&other) noexcept = default;                                                        \
    ClassName &ClassName::operator=(ClassName &&other) noexcept = default

// ------------------------------------------------------------------------------------------------------------------ //
#define ZIVID_PIMPL_REFERENCE_SEMANTICS(ClassName, Attributes)                                                         \
public:                                                                                                                \
    Attributes ~ClassName();                                                                                           \
    Attributes ClassName(const ClassName &other);                                                                      \
    Attributes ClassName &operator=(const ClassName &other);                                                           \
    Attributes ClassName(ClassName &&other) noexcept;                                                                  \
    Attributes ClassName &operator=(ClassName &&other) noexcept;                                                       \
    Attributes explicit ClassName(std::shared_ptr<class ClassName##Impl> &&impl);                                      \
    Attributes class ClassName##Impl &getImpl();                                                                       \
    Attributes const class ClassName##Impl &getImpl() const;                                                           \
                                                                                                                       \
private:                                                                                                               \
    std::shared_ptr<class ClassName##Impl> m_impl

#define ZIVID_PIMPL_REFERENCE_SEMANTICS_CPP(ClassName)                                                                 \
    ClassName::ClassName(std::shared_ptr<ClassName##Impl> &&impl)                                                      \
        : m_impl{ std::move(impl) }                                                                                    \
    {}                                                                                                                 \
                                                                                                                       \
    ClassName##Impl &ClassName::getImpl()                                                                              \
    {                                                                                                                  \
        return const_cast<ClassName##Impl &>(std::as_const(*this).getImpl());                                          \
    }                                                                                                                  \
                                                                                                                       \
    const ClassName##Impl &ClassName::getImpl() const                                                                  \
    {                                                                                                                  \
        if(m_impl == nullptr)                                                                                          \
        {                                                                                                              \
            throw std::runtime_error(#ClassName " is empty");                                                          \
        }                                                                                                              \
        return *m_impl;                                                                                                \
    }                                                                                                                  \
                                                                                                                       \
    ClassName::~ClassName() = default;                                                                                 \
    ClassName::ClassName(const ClassName &other) = default;                                                            \
    ClassName &ClassName::operator=(const ClassName &other) = default;                                                 \
    ClassName::ClassName(ClassName &&other) noexcept = default;                                                        \
    ClassName &ClassName::operator=(ClassName &&other) noexcept = default
