/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <type_traits>
#include <utility>

namespace Zivid
{
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        namespace TypeTraits
        {
            // This file is intended to be used in public headers only. Internally use
            // Traits.h exclusively (the traits in this file are available via Traits.h)

            template<typename T, typename...>
            struct IsInList : std::false_type
            {};

            template<typename T, typename First, typename... Rest>
            struct IsInList<T, First, Rest...>
                : std::integral_constant<bool, std::is_same<T, First>::value || IsInList<T, Rest...>::value>
            {};

            template<typename T, typename Tuple>
            struct IsInTuple;

            template<typename T, typename... Ts>
            struct IsInTuple<T, std::tuple<Ts...>> : IsInList<T, Ts...>
            {};

            template<typename Tuple, typename... Args>
            struct AllArgsAreInTuple : std::true_type
            {};

            template<typename Tuple, typename Arg, typename... Args>
            struct AllArgsAreInTuple<Tuple, Arg, Args...>
                : std::integral_constant<bool, IsInTuple<Arg, Tuple>::value && AllArgsAreInTuple<Tuple, Args...>::value>
            {};

            template<typename... Args>
            struct AllArgsDecayedAreUnique : std::true_type
            {};

            template<typename Head, typename... Tail>
            struct AllArgsDecayedAreUnique<Head, Tail...>
                : std::integral_constant<
                      bool,
                      !IsInList<typename std::decay<Head>::type, typename std::decay<Tail>::type...>::value
                          && AllArgsDecayedAreUnique<Tail...>::value>
            {};

            template<typename... Ts>
            struct Concatenate;

            template<typename... T1s, typename... T2s, typename... Rest>
            struct Concatenate<std::tuple<T1s...>, std::tuple<T2s...>, Rest...>
                : Concatenate<std::tuple<T1s..., T2s...>, Rest...>
            {};

            template<typename... T1s>
            struct Concatenate<std::tuple<T1s...>>
            {
                using Type = std::tuple<T1s...>;
            };

            template<typename T>
            struct TypeIdentity
            {
                using Type = T;
            };

            template<typename...>
            struct Conjunction : std::true_type
            {};

            template<typename T>
            struct Conjunction<T> : T
            {};

            template<typename T, typename... Ts>
            struct Conjunction<T, Ts...> : std::conditional<T::value, Conjunction<Ts...>, std::false_type>::type
            {};
        } // namespace TypeTraits
    } // namespace Detail
#endif
} // namespace Zivid
