/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/CameraInfo.h>
#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Detail/TypeTraits.h>
#include <Zivid/Range.h>
#include <Zivid/Resolution.h>
#include <Zivid/Settings.h>
#include <Zivid/Settings2D.h>

#define ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(T) ZIVID_CORE_EXPORT T defaultValue(Tag<T> tag, const CameraInfo &cameraInfo)

#define ZIVID_DETAIL_CAMERA_VALID_VALUES(T)                                                                            \
    ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(T);                                                                              \
    ZIVID_CORE_EXPORT std::set<T::ValueType> validValues(Tag<T> tag, const CameraInfo &cameraInfo)

#define ZIVID_DETAIL_CAMERA_VALID_RANGE(T)                                                                             \
    ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(T);                                                                              \
    ZIVID_CORE_EXPORT Range<T::ValueType> validRange(Tag<T> tag, const CameraInfo &cameraInfo)

namespace Zivid
{
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        template<typename T>
        using Tag = TypeTraits::TypeIdentity<T>;

        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::Color);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::Acquisitions);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::Processing::Filters::Reflection::Removal::Mode);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::RegionOfInterest::Box::Extents);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::RegionOfInterest::Box::PointO);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::RegionOfInterest::Box::PointA);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::RegionOfInterest::Box::PointB);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings::RegionOfInterest::Depth::Range);
        ZIVID_DETAIL_CAMERA_DEFAULT_VALUE(Settings2D::Acquisitions);

        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Acquisition::Aperture);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Acquisition::Brightness);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Acquisition::ExposureTime);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Acquisition::Gain);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Color::Balance::Blue);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Color::Balance::Green);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Color::Balance::Red);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Color::Gamma);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(
            Settings::Processing::Filters::Experimental::ContrastDistortion::Correction::Strength);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Cluster::Removal::MinArea);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Cluster::Removal::MaxNeighborDistance);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(
            Settings::Processing::Filters::Experimental::ContrastDistortion::Removal::Threshold);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Hole::Repair::Strictness);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Hole::Repair::HoleSize);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Noise::Removal::Threshold);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Outlier::Removal::Threshold);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings::Processing::Filters::Smoothing::Gaussian::Sigma);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Acquisition::Aperture);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Acquisition::Brightness);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Acquisition::ExposureTime);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Acquisition::Gain);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Processing::Color::Balance::Blue);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Processing::Color::Balance::Green);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Processing::Color::Balance::Red);
        ZIVID_DETAIL_CAMERA_VALID_RANGE(Settings2D::Processing::Color::Gamma);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings2D::Processing::Color::Experimental::Mode);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings2D::Sampling::Pixel);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings2D::Sampling::Color);

        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Diagnostics::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Engine);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Sampling::Pixel);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Sampling::Color);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Resampling::Mode);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Color::Experimental::Mode);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(
            Settings::Processing::Filters::Experimental::ContrastDistortion::Correction::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Cluster::Removal::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(
            Settings::Processing::Filters::Experimental::ContrastDistortion::Removal::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Hole::Repair::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Noise::Removal::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Noise::Suppression::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Noise::Repair::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Outlier::Removal::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Reflection::Removal::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Reflection::Removal::Mode);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::Processing::Filters::Smoothing::Gaussian::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::RegionOfInterest::Depth::Enabled);
        ZIVID_DETAIL_CAMERA_VALID_VALUES(Settings::RegionOfInterest::Box::Enabled);

        class DefaultDataModelSetter
        {
            const CameraInfo &m_cameraInfo;

        public:
            DefaultDataModelSetter(const CameraInfo &cameraInfo)
                : m_cameraInfo(cameraInfo)
            {}

            DefaultDataModelSetter(const DefaultDataModelSetter &) = delete;
            DefaultDataModelSetter(DefaultDataModelSetter &&) = delete;
            DefaultDataModelSetter &operator=(const DefaultDataModelSetter &) = delete;
            DefaultDataModelSetter &operator=(DefaultDataModelSetter &&) = delete;

            template<typename T>
            void operator()(T &dataModelNode) const;
        };

        template<typename T, typename std::enable_if_t<T::nodeType == DataModel::NodeType::group, bool> = true>
        T defaultValue(Tag<T> /* tag */, const CameraInfo &cameraInfo)
        {
            T group;
            group.forEach(DefaultDataModelSetter(cameraInfo));
            return group;
        }

        template<typename T>
        void DefaultDataModelSetter::operator()(T &dataModelNode) const
        {
            dataModelNode = defaultValue(Tag<T>{}, m_cameraInfo);
        }

        ZIVID_CORE_EXPORT void validateSettings(const Settings &settings, const CameraInfo &cameraInfo);
        ZIVID_CORE_EXPORT void validateSettings(const Settings2D &settings, const CameraInfo &cameraInfo);
    } // namespace Detail
#endif

    namespace Experimental
    {
        namespace SettingsInfo
        {
            template<typename T>
            T defaultValue(const CameraInfo &cameraInfo)
            {
                return Detail::defaultValue(Detail::Tag<T>{}, cameraInfo);
            }

            template<typename T>
            std::set<typename T::ValueType> validValues(const CameraInfo &cameraInfo)
            {
                return Detail::validValues(Detail::Tag<T>{}, cameraInfo);
            }

            template<typename T, typename Ret = Range<typename T::ValueType>>
            Ret validRange(const CameraInfo &cameraInfo)
            {
                return Detail::validRange(Detail::Tag<T>{}, cameraInfo);
            }

            /// <summary>
            /// Returns the resolution of captured frames according to the provided camera information and settings.
            /// </summary>
            /// <remarks>
            /// Note that for a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions,
            /// depending on the pixel sampling and resampling settings used in the <see cref="Settings" /> and in the
            /// <see cref="Settings::Color" />. This function returns the resolution of the 3D Point Cloud for the frame,
            /// in other words, it returns the resolution of the <see cref="PointCloud" /> returned from
            /// <see cref="Frame::pointCloud()" />. See also <see cref="Frame::frame2D()" />.
            ///
            /// If no acquisitions have been specified in the provided settings this function will throw an exception.
            /// </remarks>
            ZIVID_CORE_EXPORT Resolution resolution(const CameraInfo &cameraInfo, const Settings &settings);
        } // namespace SettingsInfo
    } // namespace Experimental
} // namespace Zivid
