/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Camera.h"

namespace Zivid
{
    /// <summary>
    /// Interface for verifying and configuring firmware
    /// </summary>
    namespace Firmware
    {
        /// <summary>A progress callback function type.</summary>
        /// <param name="progressPercentage">The progress completion percentage (0 - 100%).</param>
        /// <param name="updateStageDescription">A textual description of the progress stage.</param>
        using ProgressCallback = std::function<void(double, const std::string &)>;

        /// <summary>Check if the firmware on the camera is of the version that is required by the API.</summary>
        /// <param name="camera">A camera to check if firmware is up to date.</param>
        ZIVID_CORE_EXPORT bool isUpToDate(const Camera &camera);

        /// <summary>Update camera firmware</summary>
        /// <remarks>
        /// If the current API requires a different firmware than what is present on the camera, the
        /// firmware will be updated to this version. The function throws if the camera is connected, or if the camera
        /// is already up to date. Call <see cref="isUpToDate"/> first to check if the camera is up to date.
        /// </remarks>
        /// <param name="camera">A disconnected camera with out of sync firmware.</param>
        /// <param name="progressCallback">A callback function to track progress of update.</param>
        ZIVID_CORE_EXPORT void update(Camera &camera, const ProgressCallback &progressCallback = {});
    } // namespace Firmware
} // namespace Zivid
