/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/Detail/VisualizationExport.h"

#include <memory>
#include <string>

namespace Zivid
{
    class PointCloud;
    class Frame;

    namespace Visualization
    {
        class VisualizerImpl;

        /// <summary>
        /// Simple visualizer component for point clouds
        /// </summary>
        class Visualizer
        {
            ZIVID_PIMPL_MOVE_ONLY(Visualizer, ZIVID_VISUALIZATION_EXPORT);

        public:
            /// <summary>Constructor</summary>
            ZIVID_VISUALIZATION_EXPORT Visualizer();

            /// <summary>Show the visualization window</summary>
            ZIVID_VISUALIZATION_EXPORT void show();

            /// <summary>Hide the visualization window</summary>
            ZIVID_VISUALIZATION_EXPORT void hide();

            /// <summary>Run the event loop. Should be called to allow interaction with the point cloud</summary>
            ZIVID_VISUALIZATION_EXPORT int run();

            /// <summary>Resize the window to specified height and width</summary>
            ZIVID_VISUALIZATION_EXPORT void resize(int h, int w);

            /// <summary>Reset the camera so that the contents will fit in the window</summary>
            ZIVID_VISUALIZATION_EXPORT void resetToFit();

            /// <summary>Show the window in full screen mode</summary>
            ZIVID_VISUALIZATION_EXPORT void showFullScreen();

            /// <summary>Show the window in maximized mode</summary>
            ZIVID_VISUALIZATION_EXPORT void showMaximized();

            /// <summary>Set the window title
            ZIVID_VISUALIZATION_EXPORT void setWindowTitle(std::string title);

            /// <summary>Show a point cloud</summary>
            ZIVID_VISUALIZATION_EXPORT void show(const PointCloud &cloud);

            /// <summary>Show a frame</summary>
            ZIVID_VISUALIZATION_EXPORT void show(const Frame &frame);

            /// <summary>Enable or disable coloring of the points with their accompanying RGB colors</summary>
            ZIVID_VISUALIZATION_EXPORT void setColorsEnabled(bool enabled);

            /// <summary>Whether coloring of the points with their accompanying RGB colors is enabled</summary>
            ZIVID_VISUALIZATION_EXPORT bool colorsEnabled() const;

            /// <summary>Enable or disable meshing</summary>
            ZIVID_VISUALIZATION_EXPORT void setMeshingEnabled(bool enabled);

            /// <summary>Whether meshing is enabled</summary>
            ZIVID_VISUALIZATION_EXPORT bool isMeshingEnabled() const;

            /// <summary>Enable or disable the axis indicator</summary>
            ZIVID_VISUALIZATION_EXPORT void setAxisIndicatorEnabled(bool enabled);

            /// <summary>Whether the axis indicator is enabled</summary>
            ZIVID_VISUALIZATION_EXPORT bool isAxisIndicatorEnabled() const;
        };
    } // namespace Visualization
} // namespace Zivid
